#Example 52a
# Uniaxial extension of a cube to illustrate pole-zero law stress/strain 
# characteristics

$INC[$#INC+1] = "$example"; #include in path so it finds MyFunc.pm
use MyFunc;
$output = "./";
system "rm axial.dat"
system "rm shear.dat"

fem de coord 3,1
fem de base;r;${example}plate

fem de node;r;${example}cube
fem de elem;r;${example}cube   
fem de fibr;r;${example}cube   
fem de elem;r;${example}cube fibre
#$file = "cube"
#fem export nodes;$file as $file offset 10
#fem export elements;$file as $file offset_elem 10

fem de equa;r;${example}cube
fem de mate;r;${example}cube
fem define acti;r;${example}calcium000
fem de init;r;${example}cube
#fem de init;r;${example}cubeShear
fem de solv;r;${example}cube
#fem solve increment 1 iter 15

# Axial stress-strain relationship
$k=2;$a=0.5;$b=1; # material parameters for axial stiffness
$N = 100
$delta = ($a*2+0.2)/$N;
for $n (0..$N)
{
  print "\nSolving iteration ", $n, "\n";
  $displacement = -$a-0.1+$n*$delta;
  MyFunc::stretchCube($example,$displacement);
  fem de node;r;cubeDeformed dependent 1
  #$file = 'cubeDeformed'
  #fem export nodes;$file field as $file
  #fem export elements;$file field as $file

  fem update gauss stress fibre total_no_hydro Piola
  fem export gauss;stress yg as gauss_stress offset 0
  fem update gauss strain components fibre
  fem export gauss;strain yg as gauss_strain 
  
  $strain = MyFunc::readStrain(1);
  $stress = MyFunc::readStress(1);
  $trueStress = MyFunc::calcStress($strain, $k, $a, $b);
  MyFunc::outputValues($strain, $stress, $trueStress, "axial.dat");
  print "\nStrain: $strain   Stress: $stress  True stress: $trueStress\n";
}

# Shear stress-strain relationship
$k=1;$a=0.9;$b=1; # material parameters for shear stiffness
$N = 100
$delta = ($a*4+0.2)/$N;
for $n (0..$N)
{
  print "\nSolving iteration ", $n, "\n";
  $displacement = -2*$a-0.1+$n*$delta;
  MyFunc::shearCube($example, $displacement);
  fem de node;r;cubeDeformed dependent 1
  #$file = 'cubeDeformed'
  #fem export nodes;$file field as $file
  #fem export elements;$file field as $file

  fem update gauss stress fibre total_no_hydro Piola
  fem export gauss;stress yg as gauss_stress offset 0
  fem update gauss strain components fibre
  fem export gauss;strain yg as gauss_strain 
  
  $strain = MyFunc::readStrain(5);
  $stress = MyFunc::readStress(5);
  $trueStress = MyFunc::calcStress($strain, $k, $a, $b);
  MyFunc::outputValues($strain, $stress, $trueStress, "shear.dat");
  print "\nStrain: $strain   Stress: $stress  True stress: $trueStress\n";
}

system "gnuplot ${example}plot.gnu"

