# Note : region 1 - solid cube; region 2 - fluid flow
#        class 1 - Finite elasticity problem ; class 2 - Flow thru elastic tube

fem define para;r;cube_coro;example
fem reallocate
fem define region;r;cube_coro;example              # Defines no. of regions
fem define coordinate;r;cube1;example reg 1        # Defines coordinates for solid
fem define node;r;cube1;example reg 1              # Reads in nodal information  
fem define base;r;cube1;example reg 1              # Defines basis functions (05) 
fem define element;r;cube1;example reg 1           # Reads in element information
fem define fibre;r;cube1;example reg 1             # Reads in fibre information for solid 
fem define element;r;cube1;example fib reg 1       # Defines fibre interpolation      
fem define equation;r;cube1;example reg 1 class 1  # Defines equations for solid
fem define material;r;cube1;example reg 1 class 1  # Reads in mech properties of solid
fem define initial;r;cube1;example reg 1 class 1   # Defines initial & boundary conds.
fem define solve;r;cube1;example reg 1 class 1     # Defines solution parameters
 
fem define coordinates;r;coro;example reg 2        # Defines coordinates for fluid flow
fem define node;r;coro;example reg 2               # Reads in nodal information 
fem define;add base;r;coro;example                 # Defines basis functions (01) 
fem define field;r;coro;example reg 2              # Reads in field information 
fem define element;r;coro;example reg 2            # Reads in element information

fem define xi;r;coro;example node reg 2            # Reads in xi positions of reg 2 wrt reg 1
fem update mesh position of 2 in 1 undeform        # Updates relative positions

fem define element;r;coro;example field reg 2      # Defines field interpolation     
fem define grid;r;coro;example coronary reg 2      # Reads in spatial discretisation    
fem update grid geometry reg 2                     # Updates grid point coordinates 
fem define equation;r;coro;example reg 2 class 2   # Defines equations for fluid flow 
fem define material;r;coro;example reg 2 class 2   # Reads in phy. & mech. properties   
fem update material field material 4 no_field 1 reg 2 class 2
#                                                  # Updates unstressed radii
fem define time;r;coro;example                     # Defines boundary conditions       
fem define initial;r;coro;example reg 2 class 2    # Defines initial conditions           
fem define solve;r;coro;example reg 2 class 2      # Defines solution parameters
fem define coup;r;coro;example                     # Defines solid & fluid mech. coupling

$time_initial=0.000;
$time_final=0.005;
$time_step=0.0000008;   
$write_out_period=0.0001;

fem export nodes;solid${time_initial} as undeformed offset 500 reg 1 class 1
fem export elements;solid_undef as undeformed offset_elem 500 offset_node 500 reg 1 class 1 
fem export elements;solid_def as deformed offset_elem 600 offset_node 600 reg 1 class 1 
fem export grid;fluid as flow offset_elem 0 offset_node 0 elastic_tube reg 2 class 2
fem export points;fluid${time_initial} grid as flow offset 0 reg 2 class 2
 
for ($time=$time_initial;$time<$time_final;$time=$time+$write_out_period)      
{
$w_time=$time+$write_out_period;
fem solve restart to $write_out_period delta_t $time_step reg 1,2 class 1,2;
fem update geometry from solution reg 1;
fem export nodes;solid${w_time} as deformed offset 600 reg 1 class 1;
fem update nodes derivative 1 linear reg 2 
fem update mesh position of 2 in 1 deform
fem update grid geometry reg 2
fem export points;fluid${w_time} grid as flow offset 0 reg 2 class 2
}




