# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

fem de param;r;$example/super; #define parameters
fem de coor 3,1;               #define coordinates
fem de base;r;$example/super;  #define 1D basis function

#reads in, then exports the data points for the left lower lobe (LLL)
fem de data;r;$example/lll_data;
fem export data;artery_data as artery_data;

#reads in the nodes and elements of a portion of the arterial mesh in the LLL
fem de node;r;$example/initial_arteries;
fem de elem;r;$example/initial_arteries;

#sets up field framework for radius only (not alveolar field)
fem de mesh;c lung field only_radius_field; 

#evaluates the generations and orders of the vessels
fem evaluate order;

#this inputs a constant value of 3.650 for the first mesh element
fem update mesh geometry radius_field 1 element 1 constant 3.650;

#defines the radius of each of the vessels, using a strahler diameter ratio of 1.5997
fem update mesh geometry radius_field 1 rd_strahler ratio_diameter 1.5997;

#adds the supernumerary vessels to the existing arterial mesh
#d_ratio:defines the diameter ratio of the supernumerary vessel to the stem vessel
#sv_freq=the supernumerary frequency from each stem vessel
#diam_ratio=the Strahler based diameter ratio -> used to define diameter of each vessel
#d_max_order=the diameter of the maximum order vessel (i.e. Strahler order 17)
fem de mesh;c lung add_super d_ratio 0.3 sv_freq 1.05 arteries diameter diam_ratio 1.5997 d_max_order 30;

#exports the nodes and elements on the mesh
fem export node;lll_arteries as lll_arteries;
fem export elem;lll_arteries as lll_arteries;

#exports the stem element group (containing all stem vessels before supernumerary addition)
fem export elem;stem as stem elem stem;

#an element group containing all the supernumerary elements (called supernumerary)
#gets created in the code, this is exported for viewing:
fem export elem;supernumeraries as supernumerary elem supernumerary;


set output;cmiss_test on;
fem list nodes region all
set output off
fem quit;
