# Example b122: Solve the Luo-Rudy 2 model for a single cell, 
# represented by a single grid point

fem define para;r;test;example

#Define geometry of the problem
#  Define underlying geometry
fem define base;d
fem define node;d
fem define elem;d
#  Define grid geometry
fem define grid;d
fem update grid geometry
fem update grid metric

#Define equation to solve
#  Define monodomain with LR2 kinetics
fem define equa;r;test;example

#Define material properties for the problem

#  Define cellular material properties
fem define cell;r;lr;example
#    Define cell variants and spatially varying material parameters
fem define mate;r;test;example cell
#  Define the continuum material parameters
fem define mate;r;test;example
fem update grid material

#Define continuum initial conditions
#  Set no-flux boundary condition
fem define init;r;test;example

#Define the explicit solve, using the adams integrator
fem def solv;r;test;example

#Solve the problem

#  Set the number of threads to use when using a multiprocessing 
#  executable
set num_threads 16

#  Initialise the solve
fem solve to 0

#  Open a binary history file to store the state variables
fem open history;test write variables yqs niqslist 1..14 binary

#  Loop through time

for $HUN ( 0..2 ) 
{
for $TEN ( 0..9 ) 
{
for $UNIT ( 0..9 ) 
{


#  Solve for the current time
fem solve restart to "${HUN}${TEN}$UNIT"


#  Write the membrane potential to the history file
fem write history time "${HUN}${TEN}$UNIT" variables yqs binary
}
}
}



#  Close the history file
fem close history binary

#  Calculate the binary signal file from the history file
fem evaluate electrode;test history test from grid yqs iy 1..14 binary

#  Define UnEmap export properties
fem define export;r;test;example

#  Export the binary signal file to UnEmap
fem export signal;test electrode signal test names cell

if ($TESTING) {
    fem compare signal;test basis both masterfile $example/test comparefile test binary;
}
