# Example b124 - Single cell cubic

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}

fem define para;r;$example/test;

#Define geometry of the problem
#  Define underlying geometry
fem define base;d;
fem define node;d;
fem define elem;d;
#  Define grid geometry
fem define grid;d;
fem update grid geometry;
fem update grid metric;

#Define equation to solve
#  Define monodomain with HH kinetics
fem define equa;r;$example/cubic;

#Define material properties for the problem

#  Define cellular material properties
fem define cell;r;$example/cubic;
#  Define the continuum material parameters
fem define mate;r;$example/test;
fem update grid material;

#Define continuum initial conditions
#  Set no-flux boundary condition
fem define init;r;$example/cubic;

#Define the explicit solve, using the adams integrator
fem def solv;r;$example/test;

#Solve the problem

#  Initialise the solve
fem solve to 0;

#  Open a binary history file to store the membrane potential,
#  the h, m, and n gates and the three currents: INa, IK, and Il and
#  the stimulus current
fem open history;test write variables yqs niqslist 1 binary;

#  Loop through time
$Tend = 100;
$dt = 1.0;
for ($time=0;$time<$Tend;$time+=$dt) {
    # Solve for the current time
    fem solve restart to $time;
    # Write the membrane potential to the history file
    fem write history time $time variables yqs binary;
}

#  Close the history file
fem close history binary;

#  Calculate the binary signal file from the history file
#  for the membrane potential (YQS(1))
fem evaluate electrode;test history test from grid yqs iy 1 binary;

#  Define UnEmap export properties
fem define export;r;$example/test;

#  Define names for the signals
fem define leads;r;$example/test;

#  Export the binary signal file to UnEmap
fem export signal;test leads signal test;

# For testing we compare the generated CMISS signal files
if ($TESTING) {
    fem compare signal;test basis both masterfile $example/test comparefile test binary;
}
