# Example b211c - A tri-cubic Hermite, single element cube, using 
# grid-based FEM to solve the Noble 98 ionic current model.

# Set-up some useful variables
$output = ".";
$format = "binary";

# Define the problem size
fem define parameters;r;minimal;example;

# Define the geometry - need a tri-cubic Hermite basis for the geometry
# and a tri-linear for the fibres and grid-based FEM
fem define coordinate;r;emech;example;
fem define node;r;emech-1x1x1-cubic;example;
fem define base;r;emech-cubic-linear;example;
fem define element;r;emech-1x1x1-cubic;example;

# Need to calculate the nodal derivatives since they are not in the
# ip files
fem update node derivative 1 linear;
fem update node derivative 2 linear;
fem update node derivative 3 linear;

fem group element 1 as ALL_ELEMENTS;

# Define 0 degree fibre orientation
fem define fibre;r;emech-0deg;example;
fem define element;r;emech-1x1x1-linear;example fibre;

# Define one grid scheme which is 9x9x9 grid points
fem define grid;r;cell-9x9x9;example;
fem update grid geometry;
fem update grid metric;
fem group grid external as boundary;
# Set the stimulus site
fem group grid xi1=0 oneoff as stimulus_site;

# Define the model to use - Noble 98
fem define equation;r;n98;example;

# Define the cellular material parameters
fem define cell;r;n98;example;
fem define material;r;n98;example cell;

# Define the continuum material parameters - using a 2:1 conductivity 
# ratio (fibre:(sheet/normal))
fem define material;r;n98;example;
fem update grid material;

# Define the default no-flux boundary condition
fem define initial;r;n98;example;

# Define the integrator - implicit Adams
fem define solve;r;implicit_grid_based_fem;example;

# Export the geometry
fem export node;$output/3dcube as cube;
fem export element;$output/3dcube as cube;

# Initialise the solve
fem solve to 0;

# Grab the membrane potential index
fem inquire cell_variable Vm return_variables VM_ARRAY,VM_IDX;

# Create a history file to hold the membrane potential at all grid
# points through time
fem open history;$output/cube write variables $VM_ARRAY niqslist $VM_IDX $format;

# The time loop
for $time ( 0..5 ) {
		print "Time = $time\n";
		# Solve the model
		fem solve restart to $time;
		# Write the solution to the history file
		fem write history time $time variables $VM_ARRAY $format;
		# Export the potential field for use in CMGUI
		fem export element;$output/potential_$time field as cube;
}

# Close the history file
fem close history $format;

# Create a CMISS signal file from the history file
fem evaluate electrode;$output/Vm history $output/cube from grid $VM_ARRAY iy $VM_IDX $format;

# Export the membrane potential at all grid points to a UnEMAP signal
# file
fem define export;r;cube;example;
fem export signal;$output/Vm electrode signal $output/Vm;

