# Example b211d - Solve the Noble 98 ionic current model over a 
# distributed mesh. The stimulus is applied at one end of a 2d strip of 
# cells (grid points) and the activation wave propagates down the strip.

# Assign some useful variables
# The file format

$FORMAT = "binary"

# The signals to export ???

$VARIABLES = 1

# The finish time of the integration

$TEND = 10

# The grid points to be stimulated

$STIMULUS_SITE = "xi1=low oneoff element 1"

# The name of the output files

$OUTPUT_FILE = "n98"

# Define the model size
fem define para;r;biline;example

# Define the geometry, using 2d linear Lagrange basis functions
fem define node;r;biline;example
fem define base;r;biline;example
fem define elem;r;biline;example

# Define a 9x9 grid scheme for each of the 10 elements
fem define grid;r;biline;example

# Update the grid information
fem update grid geometry
fem update grid metric

# Group the external grid points, used to set boundary conditions
fem group grid external as boundary
# Group the grid points to be stimulated
{
  my $command = "fem group grid $STIMULUS_SITE as stimulus_site";
  cmiss $command or die "Error in cmiss command \"$command\".\n";
}

# Define the model to be used - the monodomain Noble 98 ionic 
# current model
fem define equa;r;biline;example

# Define the cellular material parameters
fem define cell;r;n98;example
fem define material;r;n98;example cell

# Define an isotropic continuum with a conductivity of 0.17 mS/mm
fem define material;r;biline;example
fem update grid material

# Define the boundary conditions
fem define init;r;biline;example

# Define the integration scheme to use
fem define solv;r;biline;example

# Initialise the solve
fem solve to 0

# Create a history file to store the required variables at all 
# grid points
fem open history;$OUTPUT_FILE write variables yq niqlist $VARIABLES $FORMAT

# Export the geometry
fem export node;$OUTPUT_FILE as biline
fem export element;$OUTPUT_FILE as biline
fem export element;numbers as biline grid_numbers

# Integrate the model through time

for $TIME ( 0..$TEND ) 
{
  # Display the current time
  print " TIME = ".sprintf("%7.1f",${TIME})." ms"

  # Solve the model for the current time
  fem solve restart to $TIME

  # Write the new variable values to the history file
  fem write history time $TIME variables yq $FORMAT

  # Export the solution field

  fem export element;${OUTPUT_FILE}."_1".sprintf("%05d",$TIME) field as biline
}


# Close the history file
fem close history $FORMAT

# Export the UnEMAP signal file
fem evaluate electrode;$OUTPUT_FILE history $OUTPUT_FILE from grid yq iy $VARIABLES $FORMAT
fem define export;r;biline;example
fem export signal;$OUTPUT_FILE electrode signal $OUTPUT_FILE
