# Example b211m - Explict Luo-Rudy phase 2 over a small, one element,
# cube. If the geometry and fields are exported, they can be viewed 
# using the draw.com command file. The signal file can be viewed using 
# UnEMAP.

# If the example path is not set, default to current directory
if (! defined $example) {
    $example = "./";
}
# Drop off the trailing / in the example path
$chopped = chop $example;
if ($chopped ne "/") {
    $example .= $chopped;
}
#Set up problem size
fem define para;r;$example/cube;

#Define geometry of the problem
fem define coor;r;$example/cube;
fem define node;r;$example/cube;
fem define base;r;$example/cube;
fem define elem;r;$example/cube;
fem define fibr;d;
fem define elem;r;$example/cube fibre;
fem define grid;r;$example/cube;
fem update grid geometry;
fem update grid metric;
#  Some useful groups
fem group grid external as boundary;
fem group grid grid 92,93,101 as stimulus_site;

#Define equation to solve
#  Define monodomain with LR2 kinetics
fem define equa;r;$example/lr;

#Define material properties for the problem

#  Define cellular material properties
fem define cell;r;$example/lr;
#    Define cell variants and spatially varying material parameters
fem define mate;r;$example/cube cell;
#  Define continuum material parameters
fem define mate;r;$example/cube;
fem update grid material;

#Define continuum initial conditions
#  Set no-flux boundary condition
fem define init;r;$example/cube;

#Define the explicit solve, using the adams integrator
fem define solv;r;$example/test;

#The geometry can be exported to CMGUI via these commands
fem export node;3dcube as cube;              #Export node geometry
fem export elem;3dcube as cube;              #Export element geometry

#Can export the cellular material parameters with the command:
fem export element;parameters as cube field cell;

#Solve the problem

#  Initialise the solve
fem solve to 0;

#  Open a binary history file to store the membrane potential (YQ(1))
fem open history;cube write variables yq niqlist 1 binary;

# Loop through time (solve for 3ms)
for $HUN ( 0..0 ) {
    for $TEN ( 0..0 ) {
        for $UNIT ( 0..5 ) {
            # Solve for the current time
            fem solve restart to "${HUN}${TEN}$UNIT";
            # Write the membrane potential to the history file
            fem write history time "${HUN}${TEN}$UNIT" variables yq binary;
            # Export the field to CMGUI
            fem export elem;gridfield_1${HUN}${TEN}$UNIT field as cube;
        }
    }
}

# Close the history file
fem close history binary;
# Calculate the binary signal file from the history file
fem evaluate electrode;cube history cube from grid binary;
# Define UnEmap export properties
fem define export;r;$example/cube;
# Export the binary signal file to UnEmap
fem export signal;cube electrode signal cube;
