# Set up for bidomain.
$VM = 1
$PHI_E = 2

# Define parameter set
fem def para;r;test;example

# Define geometry
fem def base;r;spiral;example
fem def node;r;point;example
fem def elem;r;check_eg;example

# Define grid and useful grid groups.
fem def grid;r;point;example
fem update grid geometry 
fem update grid metric 
fem group grid external as boundary

# Define bidomain equations and material properties.
#  Turn on electroporation and scale the APD
fem def equa;r;spiral_bi;example class $VM,$PHI_E

fem def mate;r;check_eg;example class $VM,$PHI_E
fem def cell;r;spiral;example class $VM,$PHI_E
fem update grid material;example class $VM,$PHI_E

# Define initial and boundary conditions.
fem def init;r;point_bi;example class $VM,$PHI_E
fem def solve;r;spiral;example class $VM
fem def solve;r;spiral;example class $PHI_E

# Define window and solve to 0.
fem define window;c
fem solve to 0 class $VM,$PHI_E
fem draw field grid 1 zmin=-110 zmax=80 class $VM

# Open a binary history file to store the membrane potential
fem open history;test write variables yq niqlist 1 class $VM

# Loop through time

for $HUN ( 0..0 ) 
{
  for $TEN ( 0..9 ) 
  {
    for $UNIT ( 0..9 ) 
    {
      fem solve restart to "${HUN}${TEN}$UNIT" class $VM,$PHI_E
      fem draw field grid 1 zmin=-110 zmax=80 class $VM
      refresh graphics

      fem write history time "${HUN}${TEN}$UNIT" variables yq
    }
  }
}

# Close the history file, calculate the signal file
# from the history file, define export properties and export.
fem close history class $VM
fem evaluate electrode;test history test from grid yq
fem define export;r;point;example
fem convert signal infile test outfile test
fem export signal;test electrode signal test

