#Example b2144
#
# This example demonstrates how to couple multiple ventriclar
#  regions by direct assembly.
#

# Problem classes
$LVENTL=1 
$RVENTL=2 
$BEMHRT=3 
$GRIDHT=4 

# Problem regions
$LVECLS=1 
$RVECLS=2 
$GRDCLS=3 
$EXTCLS=4 

# Define problem geometry
fem define para;r;minimal;example
fem define regi;r;torsoslice;example
fem define node;r;LV3;example reg $LVENTL
fem define node;r;RV3;example reg $RVENTL
fem define node;r;bemhrt3;example reg $BEMHRT
fem define node;r;gridhtr;example reg $GRIDHT
fem define base;r;linear;example
fem define elem;r;LV3;example reg $LVENTL
fem define elem;r;RV3;example reg $RVENTL
fem define elem;r;bemhrt3;example reg $BEMHRT
fem define elem;r;gridhtr;example reg $GRIDHT
fem define fibr;r;gridhtr;example reg $GRIDHT
fem define elem;r;gridhtr;example reg $GRIDHT fibre

# Define finite difference grid
fem define grid;r;gridhtr;example reg $GRIDHT
fem update grid geomet reg $GRIDHT
fem update grid metric reg $GRIDHT
fem group grid xi2=0 elem 11..20,27,28 as LVendo reg $GRIDHT
fem group grid xi2=1 elem 11,13,19,20 as RVendo1 reg $GRIDHT
fem group grid xi2=0 elem 1,3,9,10,21,22,25,26 as RVendo2 reg $GRIDHT
fem group grid grid RVendo1,RVendo2 as RVendo reg $GRIDHT
fem group grid grid LVendo,RVendo as endocardium reg $GRIDHT
fem group grid xi2=1 elem 1..10,21..26 as epicardium  reg $GRIDHT
fem group grid external as boundary reg $GRIDHT
fem group grid line 1845 xidirn 2 as stimulus reg $GRIDHT

# Create node to grid mapping
fem update node grid grid_reg $GRIDHT bem_reg $BEMHRT exclude node 77,78
fem update node geomet from grid reg $BEMHRT except node 77,78

# Define equations
fem define equa;r;LV;example reg $LVENTL class $LVECLS lock
fem define equa;r;RV;example reg $RVENTL class $RVECLS lock
fem define equa;r;alto;example reg $GRIDHT class $GRDCLS,$EXTCLS lock

# Define material parameters
fem define mate;r;LV;example reg $LVENTL class $LVECLS
fem define mate;r;RV;example reg $RVENTL class $RVECLS
fem define mate;r;alto;example reg $GRIDHT class $GRDCLS,$EXTCLS
fem define cell;r;alto;example reg $GRIDHT class $GRDCLS,$EXTCLS
fem update grid materi reg $GRIDHT class $GRDCLS,$EXTCLS

# Define initial conditions
fem define init;r;LV;example reg $LVENTL class $LVECLS
fem define init;r;RV;example reg $RVENTL class $RVECLS
fem define init;r;alto;example reg $GRIDHT class $GRDCLS,$EXTCLS

# Define right ventricle normal reversals
fem define norm;r;ventcl;example

# Define solution parameters
fem define solv;r;alto;example reg $GRIDHT class $GRDCLS
fem define solv;r;alto;example reg $GRIDHT class $EXTCLS
fem define solv;r;LV;example reg $LVENTL class $LVECLS
fem define solv;r;RV;example reg $RVENTL class $RVECLS

# Solve the passive torso problems
fem solve reg $LVENTL class $LVECLS
fem solve reg $RVENTL class $RVECLS

# Solve the bidomain problem for 5ms
fem solve activ coupled to 5 reg $GRIDHT,$LVENTL,$RVENTL class $GRDCLS,$EXTCLS,$LVECLS,$RVECLS grid endocardium cptype 4
