C Program to solve van Capelle-Durer equations. 
C Ischemic APD (112ms). VCD with Calif. mods only (for Vdot and Tdot)
C V is transmembrane potential       =YQ(nyq,1,1,nx) in CMISS
C Y is recovery variable             =YQ(nyq,3,1,nx) in CMISS
C is calcium level.                  =YQ(nyq,4,1,nx) in CMISS 
C change in recovery variable        =YQ(nyq,5,1,nx) in CMISS
C change in transmembrane potential  =YQ(nyq,6,1,nx) in CMISS
C SAC current (-ve=inwards)          =YQ(nyq,7,1,nx) in CMISS
 
      IMPLICIT NONE
      INTEGER i
      REAL*8 t,dt,V,dVdt,Y,dYdt,T_CONST     
      REAL*8 Cap,F,I0,I1,VCD_ION,Istim
      REAL*8 tt(2000),VV(2000),YY(2000),
     '  FF(2000),II0(2000),II1(2000)

      PARAMETER (Cap=1.d-6)
     
      dt  = 0.01     !time step in ms
      t   = 0.d0     !initial time
      V   =-80.d0    !initial voltage in mV
      Y   = 0.08d0   !initial excitability/recovery variable
      dYdt= 0.d0     !initial change in Y

      DO i=1,2000
        t=t+dt
        IF(i.EQ.1) THEN
          Istim=-4.d-3
        ELSE
          Istim=0.d0
        ENDIF

        IF(dYdt.GE.0.0d0) THEN    !recovery var Y is increasing
          T_CONST=0.33d0
        ELSE IF(Y.GT.0.85d0) THEN !Y decreasing & Y>0.85
          T_CONST=0.066d0 
        ELSE                      !Y decreasing & Y<0.85
!          T_CONST=3.31d0  
          T_CONST=0.066d0 
        ENDIF
        T_CONST=T_CONST*7.d0

        IF(V.LT.-80.0d0) THEN
          dYdt=-Y/T_CONST
        ELSE IF(V.GT.-60.0d0) THEN
          dYdt=(1.0d0-Y)/T_CONST
        ELSE
          dYdt=((V+80.0d0)/20.0d0-Y)/T_CONST
        ENDIF
        Y=Y+dt*dYdt

        IF(V.LT.-70.0d0) THEN
          I1=5.0d0+0.5d0*(V+70.0d0)
        ELSE IF(V.GT.0.0d0) THEN
          I1=6.0d0+0.425d0*V     
        ELSE
          I1=5.0d0+(V+70.0d0)/70.0d0
        ENDIF

        IF(V.LT.-74.3d0) THEN
          F=7.84d0+2.0d0*(V+74.3d0)
        ELSE IF(V.GT.-27.8d0) THEN
          F=-98.84d0+1.71d0*(V+27.8d0)
        ELSE            
          F=((3.837854d-3*V +0.584649d0)*V +25.31834d0)*V
     '      +235.6256d0
        ENDIF
        F=4.0d0*F

        I0=I1+F
        VCD_ION=(Y*I1+(1.0d0-Y)*I0)*1.d-6
        dVdt=-(VCD_ION+Istim)/Cap
        IF(dVdt.LT.0.d0) THEN
          I1=2.0d0*I1 !"Ischemic" APD (112ms)
          I0=I1+F
          VCD_ION=(Y*I1+(1.0d0-Y)*I0)*1.d-6
          dVdt=-(VCD_ION+Istim)/Cap
        ENDIF

        V=V+dt*dVdt

        WRITE(*,'('' i='',I5,'' t='',F7.3,''ms V='',F7.2,'
     '    //'''mV Y='',F7.3,'' F='',E10.3,'' I0='',E10.3,'
     '    //''' I1='',E10.3,'' VCD_ION='',E10.3,'' Istim='',E10.3)') 
     '    i,t,V,Y,F,I0,I1,VCD_ION,Istim

        tt(i) =t
        VV(i) =V
        YY(i) =(Y-1.d0)*1.d2
        II1(i)=I1
        FF(i) =F
        II0(i)=I0

      ENDDO !time loop

      OPEN(UNIT=3,file='vcd.data')
      DO i=1,2000,2
        WRITE(3,'(2E13.4)') tt(i),VV(i)
      ENDDO
      DO i=1,2000,2
        WRITE(3,'(2E13.4)') tt(i),YY(i)
      ENDDO
      CLOSE(unit=3)

      OPEN(UNIT=3,file='vcdi.data')
      DO i=1,2000,2
        WRITE(3,'(2E13.4)') VV(i),II1(i)
      ENDDO
      DO i=1,2000,2
        WRITE(3,'(2E13.4)') VV(i),FF(i)
      ENDDO
      DO i=1,2000,2
        WRITE(3,'(2E13.4)') VV(i),II0(i)
      ENDDO
      CLOSE(unit=3)

      STOP
      END
