#Example_c2 Solving for potential in the skull

            #This example solves for the potential on the surface of
            #the skull due to an arbitrary dipole placed inside the
            #skull.  Note that this problem requires a large parameter
            #set for CMISS. A suitable parameters.inc file is given in
            #the example directory for this example.

#Calculate the potential on the skull
fem                            #Sets the environment to FEM.
fem define parameter;r;skull_potential;example  #sets up parameters

fem define coordinates;r;;example  #defines 3D rectangular.
fem define bases;r;;example        #Define 1 bicubic Hermite BEM basis.
fem define nodes;r;;example        #Define the geometry.
fem define elements;r;;example          
fem define equation;r;;example     #Standard laplace equation.
fem define material;r;;example     #No material definition.
fem define source;r;;example       #Define an arbitrary dipole position,
#                              !  orientation and strength.
fem define initial;r;;example      #Fix the potential at the top of the
#                              !  skull to 0.0.
fem define solve;r;;example        #Define solution parameters.
fem solve                      #Calculate the potential on the skull.
fem export elem;skull_geometry as full_skull #export the geometry
#                              !  of the skull to CMGUI.
fem export node;skull_geometry as full_skull
fem export node;skull_potential as full_skull field #Export the 
#                              !  potential to CMGUI.
fem export elem;skull_potential as full_skull field

#View the potential
gfx                            #Set the environment to GFX.
gfx create window              #Create a 3D window.
gfx modify lmodel default ambient 0.3 0.3 0.3 #Add ambient light to the
#                              !  scene.
gfx modify light default direction 0.5 0.5 2.0 #Modify the direction.
gfx create light back_top colour 0.3 0.3 0.3 direction -1.0 -0.3 0.2
#                              !Create another light.
gfx create material bone ambient 0.5 0.5 0.5 diffuse 1.00 0.92 0.67 specular 0.87 0.85 0.92 shininess 0.85
#                              !Create material that looks like bone.
gfx create material line diffuse 0.00 0.00 0.00 #Create a material
#                              !  that is black.
gfx read node skull_geometry   #Read in the exported geometry data.
gfx read element skull_geometry
gfx read node skull_potential  #Read in the exported potential data.
gfx read element skull_potential
gfx create surface from full_skull as full_skull_surface with 5*5 material bone
#                              !Create a surface from the finite 
#                              !  elements named 'full_skull_surface'
#                              !  with a discretisation of 25 patches
#                              !  per element.
gfx create line from full_skull as full_skull_line with 5 material line
#                              !Create a set of lines from the finite 
#                              !  elements named 'full_skull_line'
#                              !  with a discretisation of 5 line 
#                              !  segments per element line.
gfx draw surface name full_skull_surface #Draw the surface on the 3D window.
gfx draw line name full_skull_line  #Draw the line on the 3D window.
gfx create surface from full_skull as full_skull_potential with 5*5 material bone data potential
#                              !Create a surface which is coloured
#                              !  according to potential values.
gfx draw surface name full_skull_potential #Draw the potential on the 
#                              !  3D window.
gfx set interest_point position 90.0 0.0 0.0 #Look at the center of the skull.
gfx set view_point position -500.0 0.0 0.0 #Look from a reasonable distance away
#                              !  from the skull.
gfx update                     #Display the skull.
