# This comfile solves a couple muscle fibre/whole muscle problem #
#  using dipoles.                                                #

if( !$example ) {
  $example = ".";
}

# Use small mesh (1) or refined mesh (0)
$SmallMesh = 1;

# Regions
$RectusFemoris = 1;
$MuscleFibres = 2;

# Classes
$Laplace = 1;
$Activation = 2;

# Read in the geometries
fem define para;r;$example/2regions
fem define coor;r;$example/2regions
fem define regi;r;$example/2regions
if( $SmallMesh ) {
  fem define node;r;$example/refined_rectus_femoris region $RectusFemoris
} else {
  fem define node;r;$example/refinedx2 region $RectusFemoris
}
fem define node;r;$example/fibre region $MuscleFibres
fem define base;r;$example/combined
if( $SmallMesh ) {
  fem define elem;r;$example/refined_rectus_femoris region $RectusFemoris
} else {
  fem define elem;r;$example/refinedx2 region $RectusFemoris
}
fem define elem;r;$example/fibre region $MuscleFibres

# Define the whole muscle problem
fem define equa;r;$example/muscle region $RectusFemoris class $Laplace lock
fem define mate;r;$example/muscle region $RectusFemoris class $Laplace
fem group node 1 as firstapex region $RectusFemoris
fem group node 29 as secondapex region $RectusFemoris
if( $SmallMesh ) {
  fem group node 5..28,30..109 as surfaces region $RectusFemoris
} else {
  fem group node 5..19,21..28,30..109,137..464 as surfaces region $RectusFemoris
}
fem define init;r;$example/muscle region $RectusFemoris class $Laplace
fem define solv;r;$example/muscle region $RectusFemoris class $Laplace

# Define the fibre problem
fem define grid;r;$example/fibres region $MuscleFibres
fem update grid geometry region $MuscleFibres
fem update grid metric region $MuscleFibres
fem define equation;r;$example/fibres region $MuscleFibres class $Activation
fem define material;r;$example/fibres region $MuscleFibres class $Activation
fem define cell;r;$example/fibres region $MuscleFibres class $Activation
fem update grid material region $MuscleFibres class $Activation
fem define initial;r;$example/fibres region $MuscleFibres class $Activation
fem define solve;r;$example/fibres region $MuscleFibres class $Activation

# Solve the problems
fem solve region $RectusFemoris class $Laplace
fem solve to 0 region $MuscleFibres class $Activation

foreach $step ( 0..75 ) {
  $time = $step
  fem solve restart to $time region $MuscleFibres class $Activation
  fem define sour;c grid time $time
  fem update source region $RectusFemoris class $Laplace time $time
  fem solve region $RectusFemoris class $Laplace
#  fem export elem;fibre$time field region $MuscleFibres class $Activation as fibre_MB 
#  fem export node;muscle$time field region $RectusFemoris class $Laplace as muscle_MB 
}

# Export the whole muscle information
#fem export node;muscle_MB as muscle_MB region $RectusFemoris class $Laplace
#fem export elem;muscle_MB as muscle_MB region $RectusFemoris class $Laplace
#fem export elem;field0_MB as muscle_MB region $RectusFemoris class $Laplace field

# Export the fibre information
#fem export node;fibre_MB as fibre_MB region $MuscleFibres
#fem export elem;fibre_MB as fibre_MB region $MuscleFibres
