# Set fine to 1 to make the example run longer for benchmarking.
$fine = 0;

{
  eval { require Time::HiRes };
  if( $@ )
    {
      print "warning: $@";
    }
  else
    {
      import Time::HiRes qw(time);
    }
}

sub cpu_time()
  {
    # Add user/system parent/child cpu_times.
    my $result = 0;
    map { $result += $_ } times;
    $result;
  }

sub print_timing($$$)
  {
    my ($message,$wall,$cpu) = @_;
    printf "%s: %.3fs wall, %.3fs cpu.\n", $message, $wall, $cpu;
  }

sub print_memory_use()
  {
    # Different operating systems have different data and ps uses different
    # arguments so try a series of commands in an attempt to collect the
    # information.
    system qw(ps -o vsz -p), $$;
    system qw(ps -l -p), $$;
    system qw(ps v), $$;
  }

fem define parameter;r;bifur;example      # Declares array dimensions
fem define coordinate;r;bifur;example     # Defines the coordinate system
fem define node;r;bifur;example           # Reads in nodal information
fem define base;r;bifur;example           # Defines basis function (02 funcns : 1. 1D cubic hermite 2. 1D linear)
fem define element;r;bifur;example        # Reads in element information
fem define field;r;bifur;example          # Reads in fields. Only first field (unstressed radius at nodes) is necessary for this problem
fem define element;r;bifur;example field  # Defines interpolation of field variables (linear)
# Define grid points with coronary option to discretise the spatial computational domain.
if( $fine ) {
  fem define grid;r;bifur-fine;example coronary
} else {
  fem define grid;r;bifur;example coronary
}
fem update grid geometry;
fem define equation;r;bifur;example       # Defines equations                          
fem define material;r;bifur;example       # Reads in physical & mechanical properties
fem update material field material 4 no_field 1;
#                                         # Updates unstressed radius (mat 4) from field 1
fem define time;r;periodic;example        # Define boundary conditions. Aortic press. at inlet and mean aortic press.(constant) at distal ends.
fem define initial;r;bifur;example        # Defines initial conditions
fem define solve;r;bifur;example          # Defines solution parameters

$time_initial=0.00;                       # Sets the initial time
$time_final=0.87;                         # Sets the final time
#$time_step = $fine ? 0.000025 : 0.0001;
$time_step = $fine ? 0.000025 : 0.0003;

$write_out_period=0.0435;  #0.01;                 # Sets the results write out interval

fem export points;$time_initial grid as vessel offset 0;         
fem export grid;vessel as vessel offset_node 0 elastic_tube; 

# Results output loop

($wall_setup,$cpu_setup) = (time, cpu_time);
print_timing 'setup', $wall_setup - $^T, $cpu_setup;

for ($time=$time_initial;$time<$time_final;$time=$time+$write_out_period)       
{       
#$file=sprintf "%3.2f",$time+$write_out_period;
fem solve restart to $write_out_period delta_t $time_step;
#fem export points;$file grid as vessel offset 0           
fem export points;$time grid as vessel offset 0           
}                                                                           

($wall_solve,$cpu_solve) = (time, cpu_time);
print_timing 'solve', $wall_solve - $wall_setup, $cpu_solve - $cpu_setup;
print_memory_use;
