# Example i23
# -----------

# This command file calculates the analytic magnetic field created by a one 
# loop coil with a specified current running through it in order to validate 
# magnetic field measurements made by the SQUID.

# Set the test number
$TEST=1;

# Define offsets for the first magnetic field, second magnetic field and 
# magnetic field gradients respectively
$SQUIDOFFSET_1=10000;    
$SQUIDOFFSET_2=15000;
$SQUIDOFFSET_3=20000;

# Define the output directory
if(!defined $example) 
    {  
        $example='.' ;
        $OUT="output";
    } else{
        $OUT=".";
    }

# Define the parameters, coordinates and basis functions
fem def para;r;$example/coil;
fem def coor 3,1
fem def base;r;$example/bicubic_bilinear;

# Read in the squid channel and coil positions. The coil positions are the last
# 3 data points in the .ipdata file, thus there are 19 SQUID points plus 3 coil
# positions
# Note that this data has already been rotated and translated in order to
# "line up" with the actual position of the squid
fem def data;r;$example/cmiss_squid_19 field num_field 3;

# Define the filename extension for the .exdata files
$FILENAME=sprintf("%05d",$TEST);

# Compute the magnetic field gradient at the coil positions
# This is done by placing a "virtual" coil at the centre of the squid
# and calculating the magnetic field at distances prescribed by the actual
# coil positions

# 1. Compute the magnetic field at a given height from the torso

fem eval solu mag from MFI nss 1 coil rad_coil 3.0 current -968e-6 height_coil 7.0;

# Update the data field and export the data  
fem up data field from MFI nss 1;
fem export data;$OUT/mfi_1_$FILENAME as mfi_1 field_num 1..3 field_name magnetic_1 offset $SQUIDOFFSET_1;

# 2. Compute the magnetic field at 50mm from this given height

fem eval solu mag from MFI nss 1 coil rad_coil 3.0 current -968e-6 height_coil 57.0;

# Update the data field and export this data
fem up data field from MFI nss 1;
fem export data;$OUT/mfi_2_$FILENAME as mfi_2 field_num 1..3 field_name magnetic_2 offset $SQUIDOFFSET_2;

# 3. Compute the magnetic field gradient between the two fields

fem eval solu mag from MFI nss 1 coil baseline 50.0 rad_coil 3.0 current -968e-6 height_coil 7.0;

# Update the data field and export this data
fem up data field from MFI nss 1;
fem export data;$OUT/grad_$FILENAME as mfi_grad field_num 1..3 field_name grad offset $SQUIDOFFSET_3;



